/* Die folgenden Code-Beispiele stammen aus dem Buch:
 *  Objektorientiertes Programmieren in C++
 *   Ein Tutorial fr Ein- und Umsteiger
 * von Nicolai Josuttis, Addison-Wesley Mnchen, 2001
 *
 * (C) Copyright Nicolai Josuttis 2001.
 * Diese Software darf kopiert, verwendet, modifiziert und verteilt
 * werden, sofern diese Copyright-Angabe in allen Kopien vorhanden ist.
 * Diese Software wird "so wie sie ist" zur Verfgung gestellt.
 * Es gibt keine explizite oder implizite Garantie ber ihren Nutzen.
 */
// Headerdatei fr den New-Handler
#include <new>

// Standard-Headerdateien
#include <iostream>

// Vorwrtsdeklaration
static void eigenerNewHandler();

// Reserve-Speicherplatz
static char* reserveSpeicherplatz1;
static char* reserveSpeicherplatz2;

void initNewHandler ()
{
    // je nach Bedarf Reserve-Speicherplatz anfordern
    reserveSpeicherplatz1 = new char[1000000];
    reserveSpeicherplatz2 = new char[100000];

    // New-Handler installieren
    std::set_new_handler(&eigenerNewHandler);
}

static void eigenerNewHandler ()
{
    static bool firstKiss = true;

    // - beim ersten Mal:  Reserve-Speicherplatz bereitstellen
    // - beim zweiten Mal: Ausnahme auslsen
    if (firstKiss) {
        // Programm luft weiter bis zum zweiten Aufruf
        firstKiss = false;

        // ersten Speicherplatz fr New-Handler freigeben
        delete [] reserveSpeicherplatz1;

        // Warnung auf Standard-Fehlerkanal ausgeben
        std::cerr << "Warnung: Speicherplatz bald alle" << std::endl;
    }
    else {
        // zweiten Reserve-Speicherplatz fr New-Handler freigeben
        delete [] reserveSpeicherplatz2;

        // Fehlermeldung auf Standard-Fehlerkanal ausgeben
        std::cerr << "Speicherplatz endgueltig alle" << std::endl;

        // entsprechende Exception auslsen
        throw std::bad_alloc();
    }
}
